@@ -16,9 +16,8 @@
 #include <string.h>
 #include <jansson.h>
 
-#define U_DISABLE_CURL
-#define U_DISABLE_WEBSOCKET
 #include <ulfius.h>
+#include <u_example.h>
 
 #define PORT 7437
 #define PREFIX "/sheep"
