@@ -17,10 +17,8 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#define U_DISABLE_JANSSON
-#define U_DISABLE_CURL
-#define U_DISABLE_WEBSOCKET
 #include <ulfius.h>
+#include <u_example.h>
 
 #define PORT 7876
 #define PREFIX "/stream"
