@@ -31,6 +31,21 @@
 /** Macro to avoid compiler warning when some parameters are unused and that's ok **/
 #define UNUSED(x) (void)(x)
 
+/**
+ * For using Ulfius in embedded systems
+ * Thanks to Dirk Uhlemann
+ */
+#ifdef U_WITH_FREERTOS
+  #include <FreeRTOS_Sockets.h>
+  #define  sockaddr  freertos_sockaddr
+  typedef  unsigned long int  socklen_t;
+#else
+  #ifdef U_WITH_LWIP
+    #include <lwip/sockets.h>
+  #endif // U_WITH_LWIP
+#endif // U_WITH_FREERTOS
+
+
 /**********************************
  * Internal functions declarations
  **********************************/
@@ -48,7 +63,7 @@ struct _u_endpoint ** ulfius_endpoint_match(const char * method, const char * ur
  * fills map with the keys/values defined in the url that are described in the endpoint format url
  * return U_OK on success
  */
-int ulfius_parse_url(const char * url, const struct _u_endpoint * endpoint, struct _u_map * map);
+int ulfius_parse_url(const char * url, const struct _u_endpoint * endpoint, struct _u_map * map, int check_utf8);
 
 /**
  * ulfius_set_response_header
@@ -64,6 +79,26 @@ int ulfius_set_response_header(struct MHD_Response * response, const struct _u_m
  */
 int ulfius_set_response_cookie(struct MHD_Response * mhd_response, const struct _u_response * response);
 
+/**
+ * The utf8_check() function scans the '\0'-terminated string starting
+ * at s. It returns a pointer to the first byte of the first malformed
+ * or overlong UTF-8 sequence found, or NULL if the string contains
+ * only correct UTF-8. It also spots UTF-8 sequences that could cause
+ * trouble if converted to UTF-16, namely surrogate characters
+ * (U+D800..U+DFFF) and non-Unicode positions (U+FFFE..U+FFFF). This
+ * routine is very likely to find a malformed sequence if the input
+ * uses any other encoding than UTF-8. It therefore can be used as a
+ * very effective heuristic for distinguishing between UTF-8 and other
+ * encodings.
+ *
+ * I wrote this code mainly as a specification of functionality; there
+ * are no doubt performance optimizations possible for certain CPUs.
+ *
+ * Markus Kuhn <http://www.cl.cam.ac.uk/~mgk25/> -- 2005-03-30
+ * License: http://www.cl.cam.ac.uk/~mgk25/short-license.html
+ */
+const unsigned char * utf8_check(const char * s_orig);
+
 #ifndef U_DISABLE_WEBSOCKET
 
 /**
