@@ -50,17 +50,15 @@
 static int is_websocket_data_available(struct _websocket_manager * websocket_manager) {
   int ret = 0, poll_ret = 0;
   
-  do {
-    poll_ret = poll(&websocket_manager->fds, 1, U_WEBSOCKET_USEC_WAIT);
-    if (poll_ret == -1) {
-      y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error poll websocket read");
-      websocket_manager->connected = 0;
-    } else if (websocket_manager->fds.revents & (POLLRDHUP|POLLERR|POLLHUP|POLLNVAL)) {
-      websocket_manager->connected = 0;
-    } else if (poll_ret > 0) {
-      ret = 1;
-    }
-  } while (ret == -1);
+  poll_ret = poll(&websocket_manager->fds, 1, U_WEBSOCKET_USEC_WAIT);
+  if (poll_ret == -1) {
+    y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error poll websocket read");
+    websocket_manager->connected = 0;
+  } else if (websocket_manager->fds.revents & (POLLRDHUP|POLLERR|POLLHUP|POLLNVAL)) {
+    websocket_manager->connected = 0;
+  } else if (poll_ret > 0) {
+    ret = 1;
+  }
   return ret;
 }
 
@@ -366,7 +364,7 @@ static int ulfius_read_incoming_message(struct _websocket_manager * websocket_ma
             if (header[1] & U_WEBSOCKET_MASK) {
               (*message)->has_mask = 1;
               len = read_data_from_socket(websocket_manager, masking_key, 4);
-              if (len != 4) {
+              if (len != 4 && len >= 0) {
                 y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error reading websocket for mask");
                 ret = U_ERROR;
               } else if (len < 0) {
@@ -386,7 +384,12 @@ static int ulfius_read_incoming_message(struct _websocket_manager * websocket_ma
         if (ret == U_OK) {
           payload_data = o_malloc(msg_len*sizeof(uint8_t));
           len = read_data_from_socket(websocket_manager, payload_data, msg_len);
-          if ((unsigned int)len == msg_len) {
+          if (len < 0) {
+            ret = U_ERROR_DISCONNECTED;
+          } else if ((unsigned int)len != msg_len) {
+            y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error reading websocket for payload_data");
+            ret = U_ERROR;
+          } else {
             // If mask, decode message
             (*message)->data = o_realloc((*message)->data, (msg_len+(*message)->data_len)*sizeof(uint8_t));
             if ((*message)->has_mask) {
@@ -397,11 +400,6 @@ static int ulfius_read_incoming_message(struct _websocket_manager * websocket_ma
               memcpy((*message)->data+(*message)->data_len, payload_data, msg_len);
             }
             (*message)->data_len += msg_len;
-          } else if (!len) {
-            y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error reading websocket for payload_data");
-            ret = U_ERROR;
-          } else if (len < 0) {
-            ret = U_ERROR_DISCONNECTED;
           }
           o_free(payload_data);
         }
@@ -514,8 +512,7 @@ void * ulfius_thread_websocket(void * data) {
       pthread_mutex_lock(&websocket->websocket_manager->status_lock);
       pthread_cond_broadcast(&websocket->websocket_manager->status_cond);
       pthread_mutex_unlock(&websocket->websocket_manager->status_lock);
-    }
-    if (websocket->websocket_manager->type == U_WEBSOCKET_SERVER) {
+    } else if (websocket->websocket_manager->type == U_WEBSOCKET_SERVER) {
       ulfius_clear_websocket(websocket);
     }
   } else {
@@ -646,7 +643,6 @@ static int ulfius_websocket_connection_handshake(struct _u_request * request, st
         y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error ulfius_get_next_line_from_http_response, abort parsing response");
         close(websocket->websocket_manager->tcp_sock);
         websocket->websocket_manager->tcp_sock = -1;
-        ret = U_ERROR;
         break;
       }
     } while (1);
@@ -1111,7 +1107,7 @@ int ulfius_websocket_send_fragmented_message(struct _websocket_manager * websock
                 websocket_manager->connected = 0;
               }
               if (ulfius_push_websocket_message(websocket_manager->message_list_incoming, message) != U_OK) {
-                y_log_message(Y_LOG_LEVEL_ERROR, "Error pushing new websocket message in list");
+                y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error pushing new websocket message in list");
               }
             } else {
               websocket_manager->connected = 0;
@@ -1443,6 +1439,44 @@ int ulfius_websocket_wait_close(struct _websocket_manager * websocket_manager, u
 /** Client websocket functions **/
 /********************************/
 
+static long random_at_most(long max) {
+  unsigned long
+  // max <= RAND_MAX < ULONG_MAX, so this is okay.
+  num_bins = (unsigned long) max + 1,
+  num_rand = (unsigned long) RAND_MAX + 1,
+  bin_size = num_rand / num_bins,
+  defect   = num_rand % num_bins;
+
+  long x;
+  do {
+   x = random();
+  }
+  // This is carefully written not to overflow
+  while (num_rand - defect <= (unsigned long)x);
+
+  // Truncated division is intentional
+  return x/bin_size;
+}
+
+/**
+ * Generates a random string and store it in str
+ */
+static char * rand_string(char * str, size_t str_size) {
+  const char charset[] = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
+  size_t n;
+  
+  if (str_size > 0 && str != NULL) {
+    for (n = 0; n < str_size; n++) {
+      long key = random_at_most((sizeof(charset)) - 2);
+      str[n] = charset[key];
+    }
+    str[str_size] = '\0';
+    return str;
+  } else {
+    return NULL;
+  }
+}
+
 /**
  * Set values for a struct _u_request to open a websocket
  * request must be previously initialized
@@ -1453,7 +1487,7 @@ int ulfius_set_websocket_request(struct _u_request * request,
                                   const char * websocket_protocol,
                                   const char * websocket_extensions) {
   int ret;
-  char rand_int[17] = {0}, rand_int_base64[25] = {0};
+  char rand_str[17] = {0}, rand_str_base64[25] = {0};
   size_t out_len;
   
   if (request != NULL && url != NULL) {
@@ -1475,12 +1509,12 @@ int ulfius_set_websocket_request(struct _u_request * request,
     u_map_put(request->map_header, "Content-Length", "0");
     u_map_put(request->map_header, "User-Agent", U_WEBSOCKET_USER_AGENT "/" STR(ULFIUS_VERSION));
     srand(time(NULL));
-    sprintf(rand_int, "%04d%04d%04d%04d", rand(), rand(), rand(), rand());
-    if (!o_base64_encode((unsigned char *)rand_int, 16, (unsigned char *)rand_int_base64, &out_len)) {
-      y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error o_base64_encode with the input string %s", rand_int);
+    rand_string(rand_str, 16);
+    if (!o_base64_encode((unsigned char *)rand_str, 16, (unsigned char *)rand_str_base64, &out_len)) {
+      y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error o_base64_encode with the input string %s", rand_str);
       ret = U_ERROR;
     } else {
-      u_map_put(request->map_header, "Sec-WebSocket-Key", rand_int_base64);
+      u_map_put(request->map_header, "Sec-WebSocket-Key", rand_str_base64);
       ret = U_OK;
     }
   } else {
