@@ -29,6 +29,67 @@
 #include "u_private.h"
 #include "ulfius.h"
 
+/** Define mock yder functions when yder is disabled **/
+#ifdef U_DISABLE_YDER
+int y_init_logs(const char * app, const unsigned long init_mode, const unsigned long init_level, const char * init_log_file, const char * message) {
+  (void)(app);
+  (void)(init_mode);
+  (void)(init_level);
+  (void)(init_log_file);
+  (void)(message);
+  return 1;
+}
+
+int y_set_logs_callback(void (* y_callback_log_message) (void * cls, const char * app_name, const time_t date, const unsigned long level, const char * message), void * cls, const char * message) {
+  (void)(y_callback_log_message);
+  (void)(cls);
+  (void)(message);
+  return 1;
+}
+
+void y_log_message(const unsigned long type, const char * message, ...) {
+  (void)(type);
+  (void)(message);
+}
+
+int y_close_logs() {
+  return 1;
+}
+#endif
+
+/**
+ * Fill a map with the key/values specified
+ */
+static int ulfius_fill_map_check_utf8(void * cls, enum MHD_ValueKind kind, const char * key, const char * value) {
+  char * tmp;
+  int res;
+  UNUSED(kind);
+  
+  if (cls == NULL || key == NULL) {
+    // Invalid parameters
+    y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error invalid parameters for ulfius_fill_map_check_utf8");
+    return MHD_NO;
+  } else if (utf8_check(key) == NULL && (value == NULL || utf8_check(value) == NULL)) {
+    if (u_map_get(((struct _u_map *)cls), key) != NULL) {
+      // u_map already has a value with this this key, appending value separated with a comma ',')
+      tmp = msprintf("%s,%s", u_map_get(((struct _u_map *)cls), key), (value==NULL?"":value));
+      res = u_map_put(((struct _u_map *)cls), key, tmp);
+      o_free(tmp);
+      if (res == U_OK) {
+        return MHD_YES;
+      } else {
+        return MHD_NO;
+      }
+    } else if (u_map_put(((struct _u_map *)cls), key, (value==NULL?"":value)) == U_OK) {
+      return MHD_YES;
+    } else {
+      return MHD_NO;
+    }
+  } else {
+    return MHD_YES;
+  }
+}
+
 /**
  * Fill a map with the key/values specified
  */
@@ -224,8 +285,8 @@ static void mhd_request_completed (void *cls, struct MHD_Connection *connection,
  * return MHD_NO on error
  */
 static int mhd_iterate_post_data (void * coninfo_cls, enum MHD_ValueKind kind, const char * key,
-                      const char * filename, const char * content_type,
-                      const char * transfer_encoding, const char * data, uint64_t off, size_t size) {
+                                  const char * filename, const char * content_type,
+                                  const char * transfer_encoding, const char * data, uint64_t off, size_t size) {
   
   struct connection_info_struct * con_info = coninfo_cls;
   size_t cur_size = size;
@@ -239,19 +300,27 @@ static int mhd_iterate_post_data (void * coninfo_cls, enum MHD_ValueKind kind, c
       return MHD_NO;
     }
   } else {
-    data_dup = o_strndup(data, size); // Force value to end with a NULL character
-    if (con_info->max_post_param_size > 0) {
-      if (off > con_info->max_post_param_size) {
+    if (con_info->u_instance) {
+      if (con_info->u_instance->check_utf8 && (utf8_check(key) != NULL || data == NULL || utf8_check(data) != NULL || (filename != NULL && utf8_check(filename) != NULL))) {
         return MHD_YES;
-      } else if (off + size > con_info->max_post_param_size) {
-        cur_size = con_info->max_post_param_size - off;
+      } else {
+        data_dup = o_strndup(data, size); // Force value to end with a NULL character
+        if (con_info->max_post_param_size > 0) {
+          if (off > con_info->max_post_param_size) {
+            return MHD_YES;
+          } else if (off + size > con_info->max_post_param_size) {
+            cur_size = con_info->max_post_param_size - off;
+          }
+        }
       }
+    } else {
+      return MHD_NO;
     }
     
     if (filename != NULL) {
       filename_param = msprintf("%s_filename", key);
       if (u_map_put((struct _u_map *)con_info->request->map_post_body, filename_param, filename) != U_OK) {
-        y_log_message(Y_LOG_LEVEL_ERROR, "ulfius - Error u_map_put filename value");
+        y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error u_map_put filename value");
       }
       o_free(filename_param);
     }
@@ -272,16 +341,24 @@ static int mhd_iterate_post_data (void * coninfo_cls, enum MHD_ValueKind kind, c
  * return MHD_NO on error
  */
 static int ulfius_webservice_dispatcher (void * cls, struct MHD_Connection * connection,
-                                  const char * url, const char * method,
-                                  const char * version, const char * upload_data,
-                                  size_t * upload_data_size, void ** con_cls) {
+                                         const char * url, const char * method,
+                                         const char * version, const char * upload_data,
+                                         size_t * upload_data_size, void ** con_cls) {
 
   struct _u_endpoint * endpoint_list = ((struct _u_instance *)cls)->endpoint_list, ** current_endpoint_list = NULL, * current_endpoint = NULL;
   struct connection_info_struct * con_info = * con_cls;
   int mhd_ret = MHD_NO, callback_ret = U_OK, i, close_loop = 0, inner_error = U_OK, mhd_response_flag;
 #ifndef U_DISABLE_WEBSOCKET
+  // Websocket variables
   int upgrade_protocol = 0;
   char * protocol = NULL, * extension = NULL;
+
+  // Client certificate authentication variables
+  const union MHD_ConnectionInfo * ci;
+  unsigned int listsize;
+  const gnutls_datum_t * pcert;
+  gnutls_certificate_status_t client_cert_status;
+  int ret_cert;
 #endif
   char * content_type, * auth_realm = NULL;
   struct _u_response * response = NULL;
@@ -302,10 +379,28 @@ static int ulfius_webservice_dispatcher (void * cls, struct MHD_Connection * con
   
   if (con_info->u_instance == NULL) {
     con_info->u_instance = (struct _u_instance *)cls;
-  
   }
-  
+
   if (con_info->callback_first_iteration) {
+
+#ifndef U_DISABLE_WEBSOCKET
+    ci = MHD_get_connection_info (connection, MHD_CONNECTION_INFO_GNUTLS_SESSION);
+    if (((struct _u_instance *)cls)->use_client_cert_auth && ci != NULL && ci->tls_session != NULL) {
+      if ((ret_cert = gnutls_certificate_verify_peers2(ci->tls_session, &client_cert_status)) != 0 && ret_cert != GNUTLS_E_NO_CERTIFICATE_FOUND) {
+        y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error gnutls_certificate_verify_peers2");
+      } else if (!ret_cert) {
+        pcert = gnutls_certificate_get_peers(ci->tls_session, &listsize);
+        if ((pcert == NULL) || (listsize == 0)) {
+          y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Failed to retrieve client certificate chain");
+        } else if (gnutls_x509_crt_init(&(con_info->request->client_cert))) {
+          y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Failed to initialize client certificate");
+        } else if (gnutls_x509_crt_import(con_info->request->client_cert, &pcert[0], GNUTLS_X509_FMT_DER)) {
+          y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Failed to import client certificate");
+          gnutls_x509_crt_deinit(con_info->request->client_cert);
+        }
+      }
+    }
+#endif
     con_info->callback_first_iteration = 0;
     so_client = MHD_get_connection_info (connection, MHD_CONNECTION_INFO_CLIENT_ADDRESS)->client_addr;
     con_info->has_post_processor = 0;
@@ -318,9 +413,15 @@ static int ulfius_webservice_dispatcher (void * cls, struct MHD_Connection * con
       return MHD_NO;
     }
     memcpy(con_info->request->client_address, so_client, sizeof(struct sockaddr));
-    MHD_get_connection_values (connection, MHD_HEADER_KIND, ulfius_fill_map, con_info->request->map_header);
-    MHD_get_connection_values (connection, MHD_GET_ARGUMENT_KIND, ulfius_fill_map, &con_info->map_url_initial);
-    MHD_get_connection_values (connection, MHD_COOKIE_KIND, ulfius_fill_map, con_info->request->map_cookie);
+    if (con_info->u_instance->check_utf8) {
+      MHD_get_connection_values (connection, MHD_HEADER_KIND, ulfius_fill_map_check_utf8, con_info->request->map_header);
+      MHD_get_connection_values (connection, MHD_GET_ARGUMENT_KIND, ulfius_fill_map_check_utf8, &con_info->map_url_initial);
+      MHD_get_connection_values (connection, MHD_COOKIE_KIND, ulfius_fill_map_check_utf8, con_info->request->map_cookie);
+    } else {
+      MHD_get_connection_values (connection, MHD_HEADER_KIND, ulfius_fill_map, con_info->request->map_header);
+      MHD_get_connection_values (connection, MHD_GET_ARGUMENT_KIND, ulfius_fill_map, &con_info->map_url_initial);
+      MHD_get_connection_values (connection, MHD_COOKIE_KIND, ulfius_fill_map, con_info->request->map_cookie);
+    }
     content_type = (char*)u_map_get_case(con_info->request->map_header, ULFIUS_HTTP_HEADER_CONTENT);
     
     // Set POST Processor if content-type is properly set
@@ -401,7 +502,7 @@ static int ulfius_webservice_dispatcher (void * cls, struct MHD_Connection * con
           current_endpoint = current_endpoint_list[i];
           u_map_empty(con_info->request->map_url);
           u_map_copy_into(con_info->request->map_url, &con_info->map_url_initial);
-          if (ulfius_parse_url(url, current_endpoint, con_info->request->map_url) != U_OK) {
+          if (ulfius_parse_url(url, current_endpoint, con_info->request->map_url, con_info->u_instance->check_utf8) != U_OK) {
             o_free(response);
             y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error parsing url: ", url);
             mhd_ret = MHD_NO;
@@ -669,8 +770,10 @@ static int ulfius_webservice_dispatcher (void * cls, struct MHD_Connection * con
  * return a pointer to the mhd_daemon on success, NULL on error
  * 
  */
-static struct MHD_Daemon * ulfius_run_mhd_daemon(struct _u_instance * u_instance, const char * key_pem, const char * cert_pem) {
+static struct MHD_Daemon * ulfius_run_mhd_daemon(struct _u_instance * u_instance, const char * key_pem, const char * cert_pem, const char * root_ca_perm) {
   unsigned int mhd_flags = MHD_USE_THREAD_PER_CONNECTION;
+  int index;
+
 #ifdef DEBUG
   mhd_flags |= MHD_USE_DEBUG;
 #endif
@@ -682,7 +785,7 @@ static struct MHD_Daemon * ulfius_run_mhd_daemon(struct _u_instance * u_instance
 #endif
   
   if (u_instance->mhd_daemon == NULL) {
-    struct MHD_OptionItem mhd_ops[7];
+    struct MHD_OptionItem mhd_ops[8];
     
     // Default options
     mhd_ops[0].option = MHD_OPTION_NOTIFY_COMPLETED;
@@ -697,43 +800,41 @@ static struct MHD_Daemon * ulfius_run_mhd_daemon(struct _u_instance * u_instance
     mhd_ops[2].value = (intptr_t)ulfius_uri_logger;
     mhd_ops[2].ptr_value = NULL;
     
-    mhd_ops[3].option = MHD_OPTION_END;
-    mhd_ops[3].value = 0;
-    mhd_ops[3].ptr_value = NULL;
-    
+    index = 3;
+
     if (key_pem != NULL && cert_pem != NULL) {
       // HTTPS parameters
       mhd_flags |= MHD_USE_SSL;
-      mhd_ops[3].option = MHD_OPTION_HTTPS_MEM_KEY;
-      mhd_ops[3].value = 0;
-      mhd_ops[3].ptr_value = (void*)key_pem;
+      mhd_ops[index].option = MHD_OPTION_HTTPS_MEM_KEY;
+      mhd_ops[index].value = 0;
+      mhd_ops[index].ptr_value = (void*)key_pem;
+     
+      mhd_ops[index + 1].option = MHD_OPTION_HTTPS_MEM_CERT;
+      mhd_ops[index + 1].value = 0;
+      mhd_ops[index + 1].ptr_value = (void*)cert_pem;
       
-      mhd_ops[4].option = MHD_OPTION_HTTPS_MEM_CERT;
-      mhd_ops[4].value = 0;
-      mhd_ops[4].ptr_value = (void*)cert_pem;
-      
-      mhd_ops[5].option = MHD_OPTION_END;
-      mhd_ops[5].value = 0;
-      mhd_ops[5].ptr_value = NULL;
+      index += 2;
 
-      if (u_instance->timeout > 0) {
-        mhd_ops[5].option = MHD_OPTION_CONNECTION_TIMEOUT;
-        mhd_ops[5].value = u_instance->timeout;
-        mhd_ops[5].ptr_value = NULL;
-        
-        mhd_ops[6].option = MHD_OPTION_END;
-        mhd_ops[6].value = 0;
-        mhd_ops[6].ptr_value = NULL;
+      if (root_ca_perm != NULL) {
+        mhd_ops[index].option = MHD_OPTION_HTTPS_MEM_TRUST;
+        mhd_ops[index].value = 0;
+        mhd_ops[index].ptr_value = (void *)root_ca_perm;
+
+        index++;
       }
-    } else if (u_instance->timeout > 0) {
-      mhd_ops[3].option = MHD_OPTION_CONNECTION_TIMEOUT;
-      mhd_ops[3].value = u_instance->timeout;
-      mhd_ops[3].ptr_value = NULL;
-      
-      mhd_ops[4].option = MHD_OPTION_END;
-      mhd_ops[4].value = 0;
-      mhd_ops[4].ptr_value = NULL; 
     }
+    if (u_instance->timeout > 0) {
+      mhd_ops[index].option = MHD_OPTION_CONNECTION_TIMEOUT;
+      mhd_ops[index].value = u_instance->timeout;
+      mhd_ops[index].ptr_value = NULL;
+      
+      index++;
+    }
+
+    mhd_ops[index].option = MHD_OPTION_END;
+    mhd_ops[index].value = 0;
+    mhd_ops[index].ptr_value = NULL;
+
     return MHD_start_daemon (
       mhd_flags, u_instance->port, NULL, NULL, &ulfius_webservice_dispatcher, (void *)u_instance, 
       MHD_OPTION_ARRAY, mhd_ops,
@@ -754,22 +855,11 @@ static struct MHD_Daemon * ulfius_run_mhd_daemon(struct _u_instance * u_instance
  * return U_OK on success
  */
 int ulfius_start_framework(struct _u_instance * u_instance) {
-  // Validate u_instance and endpoint_list that there is no mistake
-  if (ulfius_validate_instance(u_instance) == U_OK) {
-    u_instance->mhd_daemon = ulfius_run_mhd_daemon(u_instance, NULL, NULL);
-    
-    if (u_instance->mhd_daemon == NULL) {
-      y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error MHD_start_daemon, aborting");
-      u_instance->status = U_STATUS_ERROR;
-      return U_ERROR_LIBMHD;
-    } else {
-      u_instance->status = U_STATUS_RUNNING;
-      return U_OK;
-    }
-  } else {
-    y_log_message(Y_LOG_LEVEL_ERROR, "ulfius_start_framework - error input parameters");
-    return U_ERROR_PARAMS;
-  }
+#ifndef U_DISABLE_WEBSOCKET
+  return ulfius_start_secure_ca_trust_framework(u_instance, NULL, NULL, NULL);
+#else
+  return ulfius_start_secure_framework(u_instance, NULL, NULL);
+#endif
 }
 
 /**
@@ -782,9 +872,19 @@ int ulfius_start_framework(struct _u_instance * u_instance) {
  * return U_OK on success
  */
 int ulfius_start_secure_framework(struct _u_instance * u_instance, const char * key_pem, const char * cert_pem) {
-  // Validate u_instance and endpoint_list that there is no mistake
-  if (ulfius_validate_instance(u_instance) == U_OK && key_pem != NULL && cert_pem != NULL) {
-    u_instance->mhd_daemon = ulfius_run_mhd_daemon(u_instance, key_pem, cert_pem);
+#ifndef U_DISABLE_WEBSOCKET
+  return ulfius_start_secure_ca_trust_framework(u_instance, key_pem, cert_pem, NULL);
+#else
+  // Check parameters and validate u_instance and endpoint_list that there is no mistake
+  if (u_instance == NULL) {
+    y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - ulfius_start_secure_framework - Error, u_instance is NULL");
+    return U_ERROR_PARAMS;
+  } else if ((key_pem == NULL && cert_pem != NULL) || (key_pem != NULL && cert_pem == NULL)) {
+    y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - ulfius_start_secure_framework - Error, you must specify key_pem and cert_pem");
+    return U_ERROR_PARAMS;
+  }
+  if (ulfius_validate_instance(u_instance) == U_OK) {
+    u_instance->mhd_daemon = ulfius_run_mhd_daemon(u_instance, key_pem, cert_pem, NULL);
     
     if (u_instance->mhd_daemon == NULL) {
       y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error MHD_start_daemon, aborting");
@@ -795,10 +895,58 @@ int ulfius_start_secure_framework(struct _u_instance * u_instance, const char *
       return U_OK;
     }
   } else {
-      y_log_message(Y_LOG_LEVEL_ERROR, "ulfius_start_secure_framework - error input parameters");
+    y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - ulfius_start_secure_framework - error input parameters");
+    return U_ERROR_PARAMS;
+  }
+#endif
+}
+
+#ifndef U_DISABLE_WEBSOCKET
+/**
+ * ulfius_start_secure_ca_trust_framework
+ * Initializes the framework and run the webservice based on the parameters given using an HTTPS connection
+ * And using a root server to authenticate client connections
+ * 
+ * u_instance:    pointer to a struct _u_instance that describe its port and bind address
+ * key_pem:       private key for the server
+ * cert_pem:      server certificate
+ * root_ca_pem:   client root CA you're willing to trust for this instance
+ * return U_OK on success
+ */
+int ulfius_start_secure_ca_trust_framework(struct _u_instance * u_instance, const char * key_pem, const char * cert_pem, const char * root_ca_pem) {
+  // Check parameters and validate u_instance and endpoint_list that there is no mistake
+  if (u_instance == NULL) {
+    y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - ulfius_start_secure_ca_trust_framework - Error, u_instance is NULL");
+    return U_ERROR_PARAMS;
+  } else if ((key_pem == NULL && cert_pem != NULL) || (key_pem != NULL && cert_pem == NULL)) {
+    y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - ulfius_start_secure_ca_trust_framework - Error, you must specify key_pem and cert_pem");
+    return U_ERROR_PARAMS;
+  } else if (root_ca_pem != NULL && (key_pem == NULL || cert_pem == NULL)) {
+    y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - ulfius_start_secure_ca_trust_framework - Error, you must specify key_pem and cert_pem in addition to root_ca_pem");
+    return U_ERROR_PARAMS;
+  }
+  if (root_ca_pem != NULL) {
+    u_instance->use_client_cert_auth = 1;
+  } else {
+    u_instance->use_client_cert_auth = 0;
+  }
+  if (ulfius_validate_instance(u_instance) == U_OK) {
+    u_instance->mhd_daemon = ulfius_run_mhd_daemon(u_instance, key_pem, cert_pem, root_ca_pem);
+    
+    if (u_instance->mhd_daemon == NULL) {
+      y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error MHD_start_daemon, aborting");
+      u_instance->status = U_STATUS_ERROR;
+      return U_ERROR_LIBMHD;
+    } else {
+      u_instance->status = U_STATUS_RUNNING;
+      return U_OK;
+    }
+  } else {
+    y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - ulfius_start_secure_ca_trust_framework - error input parameters");
     return U_ERROR_PARAMS;
   }
 }
+#endif
 
 /**
  * ulfius_stop_framework
@@ -1214,13 +1362,16 @@ void ulfius_clean_instance(struct _u_instance * u_instance) {
     u_instance->bind_address = NULL;
     u_instance->default_endpoint = NULL;
 #ifndef U_DISABLE_WEBSOCKET
-    if (((struct _websocket_handler *)u_instance->websocket_handler)->pthread_init && 
-        (pthread_mutex_destroy(&((struct _websocket_handler *)u_instance->websocket_handler)->websocket_close_lock) ||
-        pthread_cond_destroy(&((struct _websocket_handler *)u_instance->websocket_handler)->websocket_close_cond))) {
-      y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error destroying websocket_close_lock or websocket_close_cond");
+    /* ulfius_clean_instance might be called without websocket_handler being initialized */
+    if ((struct _websocket_handler *)u_instance->websocket_handler) {
+        if (((struct _websocket_handler *)u_instance->websocket_handler)->pthread_init && 
+            (pthread_mutex_destroy(&((struct _websocket_handler *)u_instance->websocket_handler)->websocket_close_lock) ||
+            pthread_cond_destroy(&((struct _websocket_handler *)u_instance->websocket_handler)->websocket_close_cond))) {
+          y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error destroying websocket_close_lock or websocket_close_cond");
+        }
+        o_free(u_instance->websocket_handler);
+        u_instance->websocket_handler = NULL;
     }
-    o_free(u_instance->websocket_handler);
-    u_instance->websocket_handler = NULL;
 #endif
   }
 }
@@ -1246,6 +1397,7 @@ int ulfius_init_instance(struct _u_instance * u_instance, unsigned int port, str
     u_instance->endpoint_list = NULL;
     u_instance->default_headers = o_malloc(sizeof(struct _u_map));
     u_instance->mhd_response_copy_data = 0;
+    u_instance->check_utf8 = 1;
     if (u_instance->default_headers == NULL) {
       y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error allocating memory for u_instance->default_headers");
       ulfius_clean_instance(u_instance);
@@ -1258,6 +1410,7 @@ int ulfius_init_instance(struct _u_instance * u_instance, unsigned int port, str
     u_instance->file_upload_callback = NULL;
     u_instance->file_upload_cls = NULL;
 #ifndef U_DISABLE_WEBSOCKET
+    u_instance->use_client_cert_auth = 0;
     u_instance->websocket_handler = o_malloc(sizeof(struct _websocket_handler));
     if (u_instance->websocket_handler == NULL) {
       y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error allocating memory for u_instance->websocket_handler");
@@ -1289,3 +1442,66 @@ int ulfius_init_instance(struct _u_instance * u_instance, unsigned int port, str
 void u_free(void * data) {
   o_free(data);
 }
+
+/**
+ * The utf8_check() function scans the '\0'-terminated string starting
+ * at s. It returns a pointer to the first byte of the first malformed
+ * or overlong UTF-8 sequence found, or NULL if the string contains
+ * only correct UTF-8. It also spots UTF-8 sequences that could cause
+ * trouble if converted to UTF-16, namely surrogate characters
+ * (U+D800..U+DFFF) and non-Unicode positions (U+FFFE..U+FFFF). This
+ * routine is very likely to find a malformed sequence if the input
+ * uses any other encoding than UTF-8. It therefore can be used as a
+ * very effective heuristic for distinguishing between UTF-8 and other
+ * encodings.
+ *
+ * I wrote this code mainly as a specification of functionality; there
+ * are no doubt performance optimizations possible for certain CPUs.
+ *
+ * Markus Kuhn <http://www.cl.cam.ac.uk/~mgk25/> -- 2005-03-30
+ * License: http://www.cl.cam.ac.uk/~mgk25/short-license.html
+ */
+const unsigned char * utf8_check(const char * s_orig) {
+  const unsigned char * s = (unsigned char *)s_orig;
+  while (*s) {
+    if (*s < 0x80) {
+      /* 0xxxxxxx */
+      s++;
+    } else if ((s[0] & 0xe0) == 0xc0) {
+      /* 110XXXXx 10xxxxxx */
+      if ((s[1] & 0xc0) != 0x80 ||
+          (s[0] & 0xfe) == 0xc0) {                  /* overlong? */
+        return s;
+      } else {
+        s += 2;
+      }
+    } else if ((s[0] & 0xf0) == 0xe0) {
+      /* 1110XXXX 10Xxxxxx 10xxxxxx */
+      if ((s[1] & 0xc0) != 0x80 ||
+	  (s[2] & 0xc0) != 0x80 ||
+	  (s[0] == 0xe0 && (s[1] & 0xe0) == 0x80) ||      /* overlong? */
+	  (s[0] == 0xed && (s[1] & 0xe0) == 0xa0) ||      /* surrogate? */
+	  (s[0] == 0xef && s[1] == 0xbf &&
+	   (s[2] & 0xfe) == 0xbe)) {                      /* U+FFFE or U+FFFF? */
+        return s;
+      } else {
+        s += 3;
+      }
+    } else if ((s[0] & 0xf8) == 0xf0) {
+      /* 11110XXX 10XXxxxx 10xxxxxx 10xxxxxx */
+      if ((s[1] & 0xc0) != 0x80 ||
+	  (s[2] & 0xc0) != 0x80 ||
+	  (s[3] & 0xc0) != 0x80 ||
+	  (s[0] == 0xf0 && (s[1] & 0xf0) == 0x80) ||      /* overlong? */
+	  (s[0] == 0xf4 && s[1] > 0x8f) || s[0] > 0xf4) { /* > U+10FFFF? */
+        return s;
+      } else {
+        s += 4;
+      }
+    } else {
+      return s;
+    }
+  }
+
+  return NULL;
+}
