@@ -11,7 +11,7 @@
 #include <jansson.h>
 
 #include <check.h>
-#include "../include/ulfius.h"
+#include <ulfius.h>
 
 int callback_function_empty(const struct _u_request * request, struct _u_response * response, void * user_data) {
   return U_CALLBACK_CONTINUE;
